/*
 * reg.h
 *
 *  Created on: 2014/2/12
 *      Author: 300210
 *
 *  Please put all Himax native registers in this file
 */

#ifndef REG_H_
#define REG_H_

#include "const.h"

/*
 * global constants and variables
 */
#define CPU_FREQ	(30000000)



/*
 * System Control Unit
 */
#define SCU_BASE			0x90000000	// APB_S0
#define SCU_REG(x)			*((volatile unsigned int *)(SCU_BASE|((x)<<2)))

#define SCU_REMAP			SCU_REG(0)
#define F_scu_remap			BITS(1,0)	// 0~3: NAHB_S0PAHB_Sx洫m

#define SCU_RESET			SCU_REG(1)
#define F_scu_sys_rst_n		BITS(1,1)	// 0=system software reset
#define F_scu_cpu_rst_n		BITS(0,0)	// 0=CPU software reset

#define SCU_STANDBY			SCU_REG(2)
#define F_cpu_wakeup		BITS(1,1)
	// write 1 to request wake-up, auto clear to 0
#define F_cpu_standby		BITS(0,0)
	// write 1 to request standby, read 1 to confirm standby
	// write 0 to leave standby, set F_cpu_wakeup to wake up CPU



/*
 * Watchdog Timer
 */
#define WDT_BASE			0x90008000	// APB_S1
#define WDT_REG(x)			*((volatile unsigned int *)(WDT_BASE|((x)<<2)))

#define WDT_TIMER_CNT		WDT_REG(0)

#define WDT_CURR_CNT		WDT_REG(1)

#define WDT_ENABLE			WDT_REG(2)
#define F_wdt_reset_en	BITS(4,4)	// 1: enable reset function
#define F_wdt_timer_en	BITS(3,3)	// 1: enable timer
#define F_wdt_prescale	BITS(2,0)	// 0~7: 1, 1/2, ..., 1/256



/*
 * Timer Control
 */
#define NUM_TIMERS	4
const unsigned int TIMER_BASE[NUM_TIMERS] = { 0x90010000, 0x90040000, 0x90058000, 0x90060000 };
#define TIMER_REG(id,x)		*((volatile unsigned int *)(TIMER_BASE[(id)-1]|((x)<<2)))

#define TIMER_TIMER_CNT(id)		TIMER_REG(id,0)

#define TIMER_CURR_CNT(id)		TIMER_REG(id,1)		// read only

#define TIMER_ENABLE(id)		TIMER_REG(id,2)
#define F_timer_int_mask	BITS(5,5)	// 1: disable interrupt, 0: enable interrupt
#define F_timer_int			BITS(4,4)	// read: 1=interrupt triggered, write: 0=clear interrupt status
#define F_timer_en			BITS(3,3)	// 1=enable timer
#define F_timer_prescale	BITS(2,0)	// 0~7 = 1, 1/4, 1/8, 1/16, 1/32, 1/64, 1/128, 1/256
	#define TIMER_DIV_1		0
	#define TIMER_DIV_4		1
	#define TIMER_DIV_8		2
	#define TIMER_DIV_16	3
	#define TIMER_DIV_32	4
	#define TIMER_DIV_64	5
	#define TIMER_DIV_128	6
	#define TIMER_DIV_256	7

#define TIMER_STATUS(id)		TIMER_REG(id,3)
#define F_time_up			BITS(0,0)	// read clear and clear interrupt, 1=timer already reach 0



/*
 * PWM Control
 */
#define NUM_PWMS	4
const unsigned int PWM_BASE[NUM_PWMS] = { 0x90018000, 0x90068000, 0x90070000, 0x90078000 };
#define PWM_REG(id,x)		*((volatile unsigned int *)(PWM_BASE[(id)-1]|((x)<<2)))

#define PWM_ENABLE(id)		PWM_REG(id,0)
#define F_pwm_en		BITS(0,0)	// 1: enable pwm

#define PWM_FREQ(id)		PWM_REG(id,1)
#define F_pwm_freq		BITS(12,0)	// freq = clk_s / F_pwm_freq / 256

#define PWM_DUTY(id)		PWM_REG(id,2)
#define F_pwm_duty		BITS(7,0)



/*
 * Interrupts
 */
#define INT_BASE		0x90020000	// APB_S4
#define INT_REG(x)		*((volatile unsigned int *)(INT_BASE|((x)<<2)))

#define INT_MASK1		INT_REG(0)	// interrupt mask[31:0], 1=enable
	#define IRQ_SPI_BOOT	BITS(0,0)
	#define IRQ_TIMER		BITS(1,1)
	#define IRQ_UART_RX		BITS(2,2)
	#define IRQ_UART_TX		BITS(3,3)
	#define IRQ_TIMER2		BITS(4,4)
	#define IRQ_SPI_MASTER	BITS(5,5)
	#define IRQ_I2C_MASTER	BITS(6,6)
	#define IRQ_DDCCI_WRITE	BITS(7,7)
	#define IRQ_DDCCI_READ	BITS(8,8)

#define INT_MASK2		INT_REG(1)	// interrupt mask[63:32], 1=enable

#define INT_POL1		INT_REG(2)	// polarity[31:0], 1=reverse
#define INT_POL2		INT_REG(3)	// polarity[63:32], 1=reverse
#define INT_STATUS1		INT_REG(4)	// interrupt status[31:0], 1=triggered
#define INT_STATUS2		INT_REG(5)	// interrupt status[63:32], 1=triggered

#define INT_OUT_POL		INT_REG(6)
	#define F_int_out_pol	BITS(0,0)	// 1=high active, 0=low active



/*
 * GPIO Control
 */
#define GPIO_BASE		0x90028000	// APB_S5
#define GPIO_REG(x)		*((volatile unsigned int *)(GPIO_BASE|((x)<<2)))

#define GPIO_A_DATA		GPIO_REG(0)	// read: port A input, write: port A output
#define GPIO_A_OUTEN	GPIO_REG(1)	// port A output enable, 1=enable
#define GPIO_B_DATA		GPIO_REG(2)
#define GPIO_B_OUTEN	GPIO_REG(3)
#define GPIO_C_DATA		GPIO_REG(4)
#define GPIO_C_OUTEN	GPIO_REG(5)
#define GPIO_D_DATA		GPIO_REG(6)
#define GPIO_D_OUTEN	GPIO_REG(7)

#define GPIO_TEST_EN	GPIO_REG(8)
#define F_gpio_test_md	BITS(2,1)	// 0:B->A, 1:A->B, 2:D->C, 3:C->D
#define F_gpio_test_en	BITS(0,0)	// 1: enable test mode



/*
 * UART
 */
#define NUM_UARTS	2
const unsigned int UART_BASE[NUM_UARTS] = { 0x90030000, 0x90090000 };
#define UART_REG(id,x)		*((volatile unsigned int *)(UART_BASE[(id)-1]+((x)<<2)))

#define UART_DATA(id)		UART_REG(id,0)
// read: get one byte data from FIFO
// write: write one byte data into FIFO

#define UART_INT_EN(id)		UART_REG(id,1)	// 1=enable interrupt, 0=disable
#define F_uart_int_modem_en		BITS(3,3)	// modem status interrupt
#define F_uart_int_rxline_en	BITS(2,2)	// receiver line status interrupt
#define F_uart_int_txempty_en	BITS(1,1)	// transmitter holding register empty interrupt
#define F_uart_int_rx_en		BITS(0,0)	// received data available interrupt

#define UART_INT_ST(id)		UART_REG(id,2)	// (read only) interrupt status
#define F_uart_tx_full			BITS(4,4)	// 1=TX FIFO full
#define F_uart_fifo_only		BITS(3,3)	// 1=character reception timeout
#define F_uart_int_st			BITS(2,1)	// interrupt status
	// 3 - receiver line: overrun error, parity error, framing error, or break interrupt indication
	// 2 - received data: data more than FIFO threshold
	// 1 - tx empty: TX FIFO is empty
	// 0 - modem status: please read modem status register
#define F_uart_int_pend			BITS(0,0)	// 0=an interrupt is pending

#define UART_FIFO(id)		UART_REG(id,2)	// (write only) FIFO control, same register address as interrupt status
#define F_uart_fifo_rxth		BITS(7,6)
	// 16 byte fifo: 0~3 = 1, 4, 8, 14 characters
	// 32 byte fifo: 0~3 = 1, 8, 16, 28 characters
	// 64 byte fifo: 0~3 = 1, 16, 32, 56 characters
	// 128 byte fifo: 0~3 = 1, 32, 64, 120 characters
#define F_uart_fifo_txth		BITS(5,4)
	// 16 byte fifo: 0~3 = 1, 3, 9, 13 characters
	// 32 byte fifo: 0~3 = 1, 6, 18, 26 characters
	// 64 byte fifo: 0~3 = 1, 16, 32, 56 characters
	// 128 byte fifo: 0~3 = 1, 32, 64, 120 characters
#define F_uart_fifo_dma			BITS(3,3)	// 1=enable DMA mode in irDA
#define F_uart_fifo_txrst		BITS(2,2)	// 1=clear TX FIFO and reset counter to 0
#define F_uart_fifo_rxrst		BITS(1,1)	// 1=clear RX FIFO and reset counter to 0
#define F_uart_fifo_en			BITS(0,0)	// 1=enable both TX and RX FIFO (auto reset FIFO when set)

#define UART_LINE(id)		UART_REG(id,3)
#define F_uart_baud_reg			BITS(7,7)	// 1=uart reg 0,1,2 is using for setting baud rate
#define F_uart_setbreak			BITS(6,6)	// 1=break (pull low on TX pin)
#define F_uart_parity_bit		BITS(5,5)	// 1=force the parity bit be equal to F_uart_parity_even
#define F_uart_parity_even		BITS(4,4)	// 1=even parity, 0=odd parity
#define F_uart_parity_en		BITS(3,3)	// 1=enable parity bit
#define F_uart_stop_bit			BITS(2,2)	// 0=1-bit, 1=2-bit
#define F_uart_char_len			BITS(1,0)	// 0~3 = 5~8 bits

#define UART_LINE_ST(id)	UART_REG(id,5)	// (read only and read clear)
#define F_uart_fifo_err			BITS(7,7)	// 1=parity error, framing error or break
#define F_uart_tx_idle			BITS(6,6)	// 1=no transmission is on TX pin
#define F_uart_tx_empty			BITS(5,5)	// 1=TX FIFO is empty
#define F_uart_rx_break			BITS(4,4)	// 1=RX pin is low for one character time
#define F_uart_frame_err		BITS(3,3)	// 1=(first char in RX FIFO) stop bit is invalid
#define F_uart_parity_err		BITS(2,2)	// 1=(first char in RX FIFO) parity bit is wrong
#define F_uart_rx_full			BITS(1,1)	// 1=no place in RX FIFO to place new character
#define F_uart_rx_ready			BITS(0,0)	// 1=new data in RX FIFO

#define UART_BAUD_DIVL(id)	UART_REG(id,0)	// lower 8-bit of divisor, available if F_uart_baud_reg=1
#define UART_BAUD_DIVH(id)	UART_REG(id,1)	// higher 8-bit of divisor, available if F_uart_baud_reg=1
	// set to 0, there will be no output clock
	// UART baud rate = io_irda_uclk / 16 / PRE / DIV

#define UART_BAUD_PRE(id)	UART_REG(id,2)	// 5-bit prescaler
	// set to 0, there will be no input clock to divisor latch
	// UART baud rate = io_irda_uclk / 16 / PRE / DIV

#define UART_MODE(id)		UART_REG(id,8)
#define F_uart_mode_sel			BITS(1,0)	// 0=UART(def), 1=SIR, 2=FIR



/*
 * SPI module
 * Note: SPI1 is used to access flash for CPU and is master mode in default.
 *       The default setting is in SPI_AHB_SET.
 *       We can still test SPI1 with SPI_APB_SET.
 */
#define NUM_SPIS	2
const unsigned int SPI_BASE[NUM_SPIS] = { 0x90038000, 0x90048000 };
#define SPI_REG(id,x)		*((volatile unsigned int *)(SPI_BASE[(id)-1]+((x)<<2)))

#define SPI_APB_SET(id)		SPI_REG(id,0)
#define F_spi_baud			BITS(7,0)	// BaudRate=BusClock/((F_spi_baud+1)*2)
#define F_spi_csht			BITS(19,16)	// min time of CS high level = (F_spi_csht+1)/(BaudRate*2)
#define F_spi_spnts			BITS(20,20)	// sample point (master mode only), 0: normal, 1: delay to next edge
#define F_spi_cpha			BITS(24,24)	// clock phase, data is sampled at 0: odd edges, 1: even edges
#define F_spi_cpol			BITS(25,25)	// idle value of SCK between master and slave, 0: low, 1: high
#define F_spi_mssl			BITS(26,26)	// SPI mode, 0: master, 1: slave

#define SPI_PIO_REG(id)		SPI_REG(id,1)
#define F_spi_pio_miso		BITS(0,0)	// input value of pin MISO
#define F_spi_pio_mosi		BITS(1,1)	// I/O value of pin MOSI
#define F_spi_pio_sck		BITS(2,2)	// I/O value of pin SCK
#define F_spi_pio_cs		BITS(3,3)	// I/O value of pin CS
#define F_spi_pio_pioe		BITS(4,4)	// Programming IO Enable (only in master mode)

#define SPI_CTRL(id)		SPI_REG(id,2)
#define F_spi_reset			BITS(0,0)	// SPI reset, write 1 clear
#define F_spi_rx_reset		BITS(1,1)	// RxFIFO reset, write 1 clear
#define F_spi_tx_reset		BITS(2,2)	// TxFIFO reset, write 1 clear
#define F_spi_rx_threshold	BITS(14,10)	// RxFIFO Threshold
#define F_spi_tx_threshold	BITS(22,18)	// TxFIFO Threshold

#define SPI_STATUS(id)		SPI_REG(id,3)
#define F_spi_busy			BITS(0,0)	// SPI Transfer is active
#define F_spi_rx_empty		BITS(8,8)	// RxFIFO Empty Flag
#define F_spi_rx_full		BITS(9,9)	// RxFIFO Full Flag
#define F_spi_rx_ve			BITS(14,10)	// RxFIFO number of valid entries
#define F_spi_tx_empty		BITS(16,16)	// TxFIFO Empty Flag
#define F_spi_tx_full		BITS(17,17)	// TxFIFO Full Flag
#define F_spi_tx_ve			BITS(22,18)	// TxFIFO number of valid entries

#define SPI_INT(id)			SPI_REG(id,4)
#define F_spi_int_rx_over	BITS(0,0)	// RxFIFO overrun
#define F_spi_int_tx_under	BITS(1,1)	// TxFIFO underrun
#define F_spi_int_rx_th		BITS(2,2)	// valid entry >= RxFIFO threshold
#define F_spi_int_tx_th		BITS(3,3)	// valid entry <= TxFIFO threshold
#define F_spi_int_end		BITS(4,4)	// SPI transmit END
#define F_spi_int_txreq		BITS(5,5)	// AHB/APB TxReq conflict

#define SPI_INT_CLEAR(id)	SPI_REG(id,5)		// write 1 to clear

#define SPI_DATA_CTRL(id)	SPI_REG(id,6)
#define F_spi_read_cnt		BITS(10,0)		// RCNT+1 bytes will be received from SPI bus
#define F_spi_dummy_cnt		BITS(14,12)		// DYCNT+1 dummy bytes
#define F_spi_write_cnt		BITS(26,16)		// WCNT+1 bytes will be transmitted to SPI bus
#define F_spi_transfer_mode	BITS(30,28)
	#define SPI_WRITE				1
	#define SPI_READ				2
	#define SPI_WRITE_READ			3
	#define SPI_READ_WRITE			4
	#define SPI_WRITE_DUMMY_READ	5
	#define SPI_READ_DUMMY_WRITE	6
#define F_spi_spit	BITS(31,31)
	// SPI bus trigger: start the transmit/receive progress
	// write 1 trigger SPI to normal mode
	// read 1=SPI in normal mode, 0=the progress is completed

#define SPI_DATA(id)		SPI_REG(id,7)
	// read 4 bytes from RxFIFO, write 4 bytes to TxFIFO (LSB first)

#define SPI_AHB_SET(id)		SPI_REG(id,8)
	// share the same fields with SPI_APB_SET
	//#define F_spi_freq	// BaudRate=BusClock/((F_spi_freq+1)*2)
	//#define F_spi_csht	// min time of CS high level = (F_spi_csht+1)/(BaudRate*2)
	//#define F_spi_spnts	// sample point, 0: normal, 1: delay to next edge
	//#define F_spi_cpha	// data is sampled at 0: odd edges, 1: even edges
	//#define F_spi_cpol	// not used in only master mode
	//#define F_spi_mssl	// SPI mode, only in master mode

#define SPI_VERSION(id)		SPI_REG(id,15)
#define F_spi_year			BITS(23,16)	// 0x08 --> year 2008
#define F_spi_ver			BITS(15,0)	// 0x0100 --> v1.00



/*
 * I2C Master
 */
#define I2C_BASE		0x90050000	// APB_S10
#define I2C_REG(x)		*((volatile unsigned int *)(I2C_BASE|((x)<<2)))

#define I2C_CTRL		I2C_REG(0)
#define F_i2c_sdalow		BITS(16,16)	// 1: SDA pulled low
#define F_i2c_scllow		BITS(15,15)	// 1: SCL pulled low
#define F_i2c_int_start		BITS(14,14)	// 1: when detect a START conditon, interrupts
#define F_i2c_int_arbit		BITS(13,13)	// 1: when lose arbitration in master mode, interrupts
#define F_i2c_int_slave		BITS(12,12)	// 1: when detect a slave address matches I2C_SLAVEADDR,
#define F_i2c_int_stop		BITS(11,11)	// 1: when detect a STOP condition, interrupts
#define F_i2c_int_nack		BITS(10,10)	// 1: when detect non-ack responses, interrupts
#define F_i2c_int_input		BITS(9,9)	// 1: when I2C_DATA has received one data byte
#define F_i2c_int_output	BITS(8,8)	// 1: when I2C_DATA has transmitted one data byte
#define F_i2c_transfer_en	BITS(7,7)	// 1: ready to receive/transmit one byte, 0: make SCL=0 to stock bus
#define F_i2c_ack			BITS(6,6)	// the ack signal when reading data, 0: ACK, 1: NACK
#define F_i2c_stop			BITS(5,5)	// 1: in master mode, initiate a stop condition after next data byte
#define F_i2c_start			BITS(4,4)	// 1: in master mode, initiate a start or repeat start
#define F_i2c_gen_call		BITS(3,3)	// 1: in slave mode, respond to a general call message
#define F_i2c_master_scl	BITS(2,2)	// 1: enable SCL output for master mode
#define F_i2c_enable		BITS(1,1)	// 1: enable i2c bus interface controller
#define F_i2c_reset			BITS(0,0)	// 1: reset controller, auto cleared after 2 PCLK

#define F_i2c_transfer		merge(F_i2c_stop,merge(F_i2c_start,F_i2c_transfer_en))
	#define I2C_TX			0x8		// the offset of merged field is 4
	#define I2C_START		0x9
	#define I2C_STOP		0xA

#define I2C_STATUS		I2C_REG(1)// bit11-4 are read-clear, bit3-0 are read-only
#define F_i2c_det_start		BITS(11,11)	// 1: detected a START condition
#define F_i2c_det_arbit		BITS(10,10)	// 1: controller lost arbitration in master mode
#define F_i2c_det_gencall	BITS(9,9)	// 1: in slave mode, received a slave address matches general call address
#define F_i2c_det_slave		BITS(8,8)	// 1: in slave mode, received a slave address matches I2C_SLAVEADDR
#define F_i2c_det_stop		BITS(7,7)	// 1: detected a STOP condition
#define F_i2c_det_nack		BITS(6,6)	// 1: in master mode, detected a NACK reponse
#define F_i2c_det_input		BITS(5,5)	// 1: I2C_DATA has new data byte from i2c bus
#define F_i2c_det_output	BITS(4,4)	// 1: the data in I2C_DATA has been sent
#define F_i2c_bus_busy		BITS(3,3)	// 1: bus is busy, controller is not involved in the transaction
#define F_i2c_busy			BITS(2,2)	// 1: controller is busy (between START and STOP)
#define F_i2c_got_nack		BITS(1,1)	// 1: controller receives or sends NACK
#define F_i2c_readwrite		BITS(0,0)	// 1: controller is in master-receive or slave-transmit mode

	#define I2C_SLAVEADDR_MASK	0x120
	#define I2C_SLAVEADDR_OK	0x120

	#define I2C_SLAVEWR_MASK	0x92
	#define I2C_SLAVEWR_OK		0x10

	#define I2C_SLAVERD_MASK	0xA2
	#define I2C_SLAVERD_OK		0x20

	#define I2C_MASTERWR_MASK	0x212
	#define I2C_MASTERWR_OK		0x010

	#define I2C_MASTERRD_MASK	0x20
	#define I2C_MASTERRD_OK		0x20

#define I2C_CLOCK			I2C_REG(2)	// SCL_Freq = CLK_S / (2*I2C_CLOCK[17:0] + 4 + GSR)

#define I2C_DATA			I2C_REG(3)

#define I2C_SLAVEADDR		I2C_REG(4)
#define F_i2c_10bit_en	BITS(31,31)	// enable 10-bit address
#define F_i2c_addr_10	BITS(9,0)	// slave address in 10-bit mode
#define F_i2c_addr		BITS(6,0)	// slave address in 7-bit mode

#define I2C_TIMING			I2C_REG(5)
#define F_i2c_gsr		BITS(12,10)	// glitch supression logic period: GSR*PCLK
#define F_i2c_tsr		BITS(9,0)	// the delay between SCL=0 and data/ack = GSR+TSR+4

#define I2C_BUS_MONITOR		I2C_REG(6)
#define F_i2c_sclin		BITS(1,1)	// reflects SCLin pin
#define F_i2c_sdain		BITS(0,0)	// reflects SDAin pin



/*
 * DDC/CI
 */
#define DDC_BASE			0x80000000	// AHB_S8
#define DDC_REG(x)			*((volatile unsigned int *)(DDC_BASE|((x)<<2)))

#define DDC_ENABLE			DDC_REG(0)
#define F_ddc_en		BITS(0,0)	// 1=enable DDCCI
#define F_ddc_ready		BITS(1,1)	// 1=ready for read out by I2C interface
#define F_ddc_wint_en	BITS(2,2)	// 1=enable write buffer interrupt
#define F_ddc_rint_en	BITS(3,3)	// 1=enable read buffer interrupt

#define DDC_INT_STATUS		DDC_REG(1)
#define F_ddc_wint		BITS(0,0)	// 1=write interrupt occurs, write 1 to clear
#define F_ddc_rint		BITS(1,1)	// 1=read interrupt occurs, write 1 to clear

#define DDC_WBUF			DDC_REG(2)
#define F_ddc_wbuf_addr	BITS(5,0)	// address of write buffer
#define F_ddc_wbuf_data	BITS(15,8)	// data of write buffer (read only)

#define DDC_RBUF			DDC_REG(3)
#define F_ddc_rbuf_addr	BITS(5,0)	// address of read buffer
#define F_ddc_rbuf_data	BITS(15,8)	// data of read buffer


#endif /* REG_H_ */
