/*
 * reg_obj.cpp
 *
 *  Created on: 2014/4/7
 *      Author: 300210
 */

#include <limits.h>		// for Timer
#include <stdarg.h>		// for UART printf
#include "reg_obj.h"

bool Timer::Init(uint myID)
{
	if (myID > 4 || myID < 1) {
		myID = 100;		// ]w@ӵΤ쪺
		return false;
	}
	id = myID;

	for (int sid=0; sid<5; sid++) {
		event[sid].function = 0;
	}

	setbit(TIMER_ENABLE(id), F_timer_prescale, TIMER_DIV_16);	// prescale = 1/16
	return true;
}

/*
 * \Gwתɶ
 * milli_seconds: 𪺮ɶ
 */
void Timer::Delay1ms(uint milli_seconds)
{
	if (id > 4) return;

	TIMER_TIMER_CNT(id) = ms_count * milli_seconds;
	setbit(TIMER_ENABLE(id), F_timer_en, 1);
	while (getbit(TIMER_STATUS(id),F_time_up) == 0) {
		/* Empty Body */
	}
	setbit(TIMER_ENABLE(id), F_timer_en, 0);
	return;
}

/*
 * \GsW@Өƥ
 * myfunc: ƥoͫn檺禡
 * type: PERIOD_TIMER: gʨƥ, ONCE_TIMER: @ʨƥ
 * timeoutMS: jh[nƥ
 */
bool Timer::EventAdd (void (*myfunc)(), int type, int timeoutMS)
{
	if (id > 2) return false;	// only Timer1/2 support events

	int sid;
	for (sid=0; sid<5; sid++) {
		if (event[sid].function == 0) break;
	}
	if (sid == 5) return false;	// there's no place to add new sub-timer

	int passed_time = TIMER_CURR_CNT(id) / ms_count;

	event[sid].function = myfunc;
	event[sid].interval_ms = timeoutMS;
	event[sid].type = type;
	event[sid].remain_ms = timeoutMS + passed_time;

	if (passed_time > 0)
	{
		for (sid=0; sid<5; sid++) {
			if (event[sid].function == 0) continue;

			event[sid].remain_ms -= passed_time;
			// this case should seldom happen
			if (event[sid].remain_ms <= 0) {
				event[sid].function();	// time's up, execute timer function
				if (event[sid].type == PERIOD_TIMER) {
					event[sid].remain_ms = event[sid].interval_ms;
				} else {
					event[sid].function = 0;	// remove this timer
				}
			}
		}
	}

	int min_remain;
	for (min_remain=INT_MAX, sid=0; sid<5; sid++) {
		if (event[sid].function == 0) continue;
		if (event[sid].remain_ms < min_remain) {
			min_remain = event[sid].remain_ms;
		}
	}

	// enable interrupt
	TIMER_TIMER_CNT(id) = ms_count * min_remain;
	setbit(TIMER_ENABLE(id), F_timer_int_mask, 0);
	setbit(TIMER_ENABLE(id), F_timer_en, 1);
	setbit(TIMER_ENABLE(id), F_timer_int, 0);

	if (id == 1) {
		setbit(INT_MASK1, IRQ_TIMER, 1);
	} else {
		setbit(INT_MASK1, IRQ_TIMER2, 1);
	}
	return true;
}

/*
 * \Gwƥ
 * myfunc: MP禡ƥAòƥ
 */
bool Timer::EventRemove (void (*myfunc)())
{
	if (id > 2) return false;	// only Timer1/2 support events

	int sid;
	for (sid=0; sid<5; sid++) {
		if (event[sid].function == myfunc) break;
	}

	if (sid < 5) {
		event[sid].function = 0;
		return true;
	}
	return false;
}

/*
 * \GTimer1/2 _oͫA|Is禡BzƥAñNҦƥ󪺳ѾlɶgLɶ
 */
void Timer::HandleEvent ()
{
	if (id > 2) return;		// only Timer1/2 support events

	int min_remain;
	int sid;

	for (min_remain=INT_MAX, sid=0; sid<5; sid++) {
		if (event[sid].function == 0) continue;
		if (event[sid].remain_ms < min_remain) {
			min_remain = event[sid].remain_ms;
		}
	}

	for (sid=0; sid<5; sid++) {
		if (event[sid].function == 0) continue;

		event[sid].remain_ms -= min_remain;
		if (event[sid].remain_ms <= 0) {
			event[sid].function();	// time's up, execute timer function
			if (event[sid].type == PERIOD_TIMER) {
				event[sid].remain_ms = event[sid].interval_ms;
			} else {
				event[sid].function = 0;	// remove this timer
			}
		}
	}

	bool at_least_one_timer_alive = false;
	for (min_remain=INT_MAX, sid=0; sid<5; sid++) {
		if (event[sid].function == 0) continue;

		at_least_one_timer_alive = true;
		if (event[sid].remain_ms < min_remain) {
			min_remain = event[sid].remain_ms;
		}
	}

	if (at_least_one_timer_alive)
	{
		TIMER_TIMER_CNT(id) = ms_count * min_remain;
		setbit(TIMER_ENABLE(id), F_timer_int_mask, !at_least_one_timer_alive);
		setbit(TIMER_ENABLE(id), F_timer_en, at_least_one_timer_alive);
		setbit(TIMER_ENABLE(id), F_timer_int, 0);
	}

	if (id == 1) {
		setbit(INT_MASK1, IRQ_TIMER, at_least_one_timer_alive);
	} else {
		setbit(INT_MASK1, IRQ_TIMER2, at_least_one_timer_alive);
	}

	// when the timer hits zero, we restart it again
	setbit(TIMER_ENABLE(id), F_timer_int, 0);
}

/*
 * \G]wgL@qɶoͤ_A]{bTimer1/2OHeventҦDAҥH禡Τ
 * enable: 1=}_, 0=_
 * msToTrigger: gLXms|Ĳo_
 */
void Timer::Interrupt_En (bool enable, uint msToTrigger)
{
	if (id > 2) return;		// only Timer1/2 can trigger interrupt

	TIMER_TIMER_CNT(id) = ms_count * msToTrigger;
	setbit(TIMER_ENABLE(id), F_timer_int_mask, !enable);
	setbit(TIMER_ENABLE(id), F_timer_en, enable);

	if (id == 1) {
		setbit(INT_MASK1, IRQ_TIMER, enable);
	} else {
		setbit(INT_MASK1, IRQ_TIMER2, enable);
	}
}




/*
 * \GlUARTó]wbaud rate
 * myID: nϥΪUARTs
 * setBaudRate: nϥΪbaud rate
 */
bool UART::Init(uint myID, uint setBaudRate)
{
	uint reg;

	if (myID > 2 || myID < 1) return false;
	id = myID;

	SetBaudRate (setBaudRate);
	SetProperties (0, 0, 3);

	reg = 0;
	setbit(reg, F_uart_fifo_txrst, 1);	// clear tx fifo
	setbit(reg, F_uart_fifo_rxrst, 1);	// clear rx fifo
	setbit(reg, F_uart_fifo_en, 1);		// enable tx and rx fifo
	UART_FIFO(id) = reg;

	head = tail = 0; // initialize queue pointer

	UART_INT_EN(id) = 0; // disable all interrupts in default
	return true;
}

/*
 * parity: 0=disable, 1=odd, 2=even (default=0)
 * stop: 0=1-bit, 1=2-bit (default=0)
 * charlen: 0=5-bit, 1=6-bit, 2=7-bit, 3=8-bit (default=3)
 */
void UART::SetProperties(int parity, int stop, int charlen)
{
	uint reg = UART_LINE(id);

	if (parity == 0) {
		setbit(reg, F_uart_parity_en, 0);	// disable parity bit
	} else {
		setbit(reg, F_uart_parity_en, 1);	// enable parity bit
	}

	if (parity == 2) {
		setbit(reg, F_uart_parity_even, 1);	// even parity
	} else {
		setbit(reg, F_uart_parity_even, 0);	// odd parity
	}

	setbit(reg, F_uart_stop_bit, stop);		// stop bit length
	setbit(reg, F_uart_char_len, charlen);	// character length
	UART_LINE(id) = reg;
}

/*
 * newBaudRate: ]wsbaud rate
 */
void UART::SetBaudRate(uint newBaudRate)
{
	baudRate = newBaudRate;
	int div = CPU_FREQ / 16 / baudRate;
	setbit(UART_MODE(id), F_uart_mode_sel, 0);		// ensure it's UART mode
	setbit(UART_LINE(id), F_uart_baud_reg, 1);		// enable baud rate setting mode
	UART_BAUD_DIVL(id) = div % 256;
	UART_BAUD_DIVH(id) = div / 256;
	UART_BAUD_PRE(id) = 1;
	setbit(UART_LINE(id), F_uart_baud_reg, 0);		// after setting, disable baud rate mode
}

/*
 * Xr
 */
void UART::puts(const char *str)
{
	while (*str)
	{
		if (*str == '\n') {
			putc('\r');
		}
		putc(*str++);
	}
}

void UART::putc(char ch)
{
	while (getbit(UART_INT_ST(id),F_uart_tx_full)) {
		/* Empty Body */
	}
	UART_DATA(id) = ch;
}

/*
 * qqueueo@Ӧr
 */
char UART::getc()
{
	if (tail == head) return 0;
	char ch = queue[tail];
	tail = (tail + 1) % QUEUE_SIZE;
	return ch;
}

/*
 * NUARTfifoeƻsqueue
 */
void UART::Enqueue()
{
	while (getbit(UART_LINE_ST(id), F_uart_rx_ready))
	{
		queue[head] = UART_DATA(id);
		// Note: if (head == tail), the queue would be seen as empty, the data would be discarded
		head = (head + 1) % QUEUE_SIZE;
	}
}

/*
 * enable: }UART RX_
 */
void UART::RxInt_En(bool enable)
{
	if (enable)
	{
		setbit(UART_INT_EN(id), F_uart_int_rx_en, 1);
		setbit(INT_MASK1, IRQ_UART_RX, 1);
	}
	else
	{
		setbit(UART_INT_EN(id), F_uart_int_rx_en, 0);
		setbit(INT_MASK1, IRQ_UART_RX, 0);
	}
}

/*
 * 䴩HU榡: %d, %u, %x, %X, %s, %c
 */
void UART::printf (const char *fmt, ...)
{
	va_list ap;
	char c, ch, *ch_ptr;
	int  i, zeroextend, desc, length, len, negative;
	uint uint_value;
	char buf[20];

	va_start(ap, fmt);

	while (*fmt)
	{
		c = *fmt++;
		if (c != '\n' && c != '%') {
			putc(c);
			continue;
		}

		if (c == '\n') {
			putc('\r');
			putc('\n');
			continue;
		}

		// processing '%' symbol
		for (i=0, desc=0, zeroextend=0, length=0; *fmt; i++)
		{
			c = *fmt++;
			switch (c)
			{
			case 'c':	desc = 1;	break;	// char
			case 'd':	desc = 2;	break;	// decimal integer
			case 's':	desc = 3;	break;	// string
			case 'u':	desc = 4;	break;	// unsigned decimal integer
			case 'x':	desc = 5;	break;	// hexidecimal integer
			case 'X':	desc = 6;	break;	// hexidecimal integer with uppercase
			case '0':
				if (i == 0)
					zeroextend = 1;
				else
					length *= 10;
				break;
			case '1':
			case '2':
			case '3':
			case '4':
			case '5':
			case '6':
			case '7':
			case '8':
			case '9':
				length *= 10;
				length += c-'0';
				break;
			}
			if (desc) break;	// if format specifier is found, exit loop
		}	// end of for

		if (desc == 0) continue;

		switch(desc)
		{
		case 1:	// %c
			ch = (char)va_arg(ap, int);
			putc(ch);
			break;

		case 2:	// %d
		case 4:	// %u
			uint_value = va_arg(ap, unsigned int);
			negative = 0;
			len = 0;

			if (desc == 2) {		// signed
				int int_value = (int)uint_value;
				if (int_value < 0) {
					negative = 1;
					uint_value = -int_value;
				}
			}
			for (len=0; uint_value; len++) {
				buf[len] = '0' + uint_value % 10;
				uint_value /= 10;
			}
			if (zeroextend && length > len) {
				for (; length>len; length--) putc('0');
			}
			if( len == 0 ) {
				putc('0');
			} else {
				if (negative) putc('-');
				for (len--; len>=0; len--) putc(buf[len]);
			}
			break;

		case 3:	// %s
			ch_ptr = va_arg(ap, char *);
			puts(ch_ptr);
			break;

		case 5:	// %x
		case 6:	// %X
			int shift;
			unsigned char nibble;
			char hex_start = (desc == 5) ? 'a' : 'A';
			len = 0;
			uint_value = va_arg(ap,unsigned int);

			for (shift=28; shift>=0; shift-=4) {
				nibble = (uint_value >> shift) & 0xf;
				if (len==0 && nibble!=0) {
					len = (shift >> 2)+1;
				}
				if (nibble < 0x0a) {
					buf[shift >>2] = ('0'+ nibble);
				} else {
					buf[shift >>2] = (hex_start + nibble - 0x0a);
				}
			}
			if (zeroextend && length>len) {
				for (; length>len; length--) putc('0');
			}
			if (len == 0 && !zeroextend) {
				putc('0');
			} else {
				for (; len>0; len--) putc(buf[len-1]);
			}
			break;
		}	// end of switch(desc)
	}	// end of while(fmt)
	va_end(ap);
}




/*
 * ]wn}ɶAprescale=16
 */
void Watchdog::Init(uint timeout)
{
	WDT_TIMER_CNT = CPU_FREQ / 16 / 1000 * timeout;
	WDT_ENABLE = TIMER_DIV_16;
}

/*
 * }lwatchdog timer
 */
void Watchdog::Start()
{
	setbit(WDT_ENABLE, F_wdt_reset_en, 1);
	setbit(WDT_ENABLE, F_wdt_timer_en, 1);
}

/*
 * nmwatchdog timerؤkAĤ@جO}timerAĤGجO]@timer count
 * o̧ڭ̥βĤ@
 * ĤGءGWDT_TIMER_CNT = WDT_TIMER_CNT;
 */
void Watchdog::Reset()
{
	setbit(WDT_ENABLE, F_wdt_timer_en, 0);
	setbit(WDT_ENABLE, F_wdt_timer_en, 1);
}



void PWM::Set(uint freq_hz, uint duty_on_255)
{
	if (freq_hz <= MaxFreq && freq_hz >= MinFreq) {
		freq_hz = 1000;
	}

	PWM_FREQ(id) = CPU_FREQ / 256 / freq_hz;
	PWM_DUTY(id) = duty_on_255;
	Enable(true);
}

bool PWM::Init (uint myID, uint freq_hz, uint duty_on_255)
{
	if (myID < 1 || myID > 4) return false;

	id = myID;
	Set (freq_hz, duty_on_255);
	Enable(true);
	return true;
}

void PWM::Enable(bool en)
{
	setbit(PWM_ENABLE(id), F_pwm_en, en);
}




bool DDCCI::Init ()
{
	uint reg = 0;
	setbit(reg, F_ddc_en, 1);
	setbit(reg, F_ddc_wint_en, 1);
	setbit(reg, F_ddc_rint_en, 1);
	DDC_ENABLE = reg;

	ReadInt(false);
	WriteInt(true);
	return true;
}

/*
 * enable: }Ū_APCŪƮɷ|Ĳo
 */
void DDCCI::ReadInt(bool enable)
{
	setbit(DDC_ENABLE, F_ddc_rint_en, enable);
	setbit(INT_MASK1, IRQ_DDCCI_READ, enable);
}

/*
 * enable: }gJ_APCݼgJƮɷ|Ĳo
 */
void DDCCI::WriteInt(bool enable)
{
	setbit(DDC_ENABLE, F_ddc_wint_en, enable);
	setbit(INT_MASK1, IRQ_DDCCI_WRITE, enable);
}

/*
 * ȮɱNDDC/CI parsero̡AHpGӪA]iHWߦ@Ө禡
 * parserݭnDDC󪺤䴩
 */
void DDCCI::ParseWbuf()
{
	for (int i=0; i<64; i++)
	{
		DDC_WBUF = i;
		DDC_RBUF = DDC_WBUF;
		UART1.printf("%02x, ", getbit(DDC_WBUF, F_ddc_wbuf_data));
	}
	UART1.puts("\n");

	// ʲM_AA_h|ױ᭱_
	setbit(DDC_INT_STATUS, F_ddc_wint, 1);
	setbit(DDC_ENABLE, F_ddc_ready, 1);
}




/*
 * myID: 0=SPI1, 1=SPI2, 䤤SPI1|QMCUΨŪcode
 * slaveMode: 1=slave, 0=master
 * myBaudRate: ]wt(Hz)
 */
uint SPI_Module::Init(uint myID, bool slaveMode, uint myBaudRate)
{
	uint reg, baud;
	baud = CPU_FREQ / myBaudRate / 2 - 1;
	baudRate = CPU_FREQ / ((baud+1) * 2);
	id = myID;
	isSlave = slaveMode;

	reg = 0;
	setbit(reg, F_spi_baud, baud);
	setbit(reg, F_spi_csht, 2);
	setbit(reg, F_spi_mssl, slaveMode);
	SPI_APB_SET(id) = reg;

	setbit(SPI_CTRL(id), F_spi_rx_threshold, 1);
	setbit(SPI_CTRL(id), F_spi_tx_threshold, 1);

	if (id == 2) {
		setbit(INT_MASK1, IRQ_SPI_MASTER, 1);
		setbit(SPI_INT(id), F_spi_int_rx_th, 1);	// if valid entry >= threshold
	}
	return baudRate;
}

/*
 * _oͫӲMAA_h|ױ᪺_
 */
void SPI_Module::ClearInt()
{
	SPI_INT_CLEAR(id) = SPI_INT_CLEAR(id);
}

/*
 * gJƨFIFOApGFIFOF|۰ʸX
 */
int SPI_Module::WriteFIFO(const char *data, int len)
{
	uint reg;
	int i;
	for (i=0; i<len; i+=4)
	{
		reg = uchar(data[i]) | (uchar(data[i+1]) << 8) |
				(uchar(data[i+2]) << 16) | (uchar(data[i+3]) << 24);
		if (getbit(SPI_STATUS(id), F_spi_tx_full)) {
			return i;
		}
		SPI_DATA(id) = reg;
	}
	return i;
}

/*
 * qFIFOŪơAöǦ^Ū쪺(@w4)
 */
int SPI_Module::Read(char *indata, int len)
{
	uint reg, timeout;
	int i;
	for (i=0; i<len; i+=4)
	{
		for (timeout=0; timeout<100 && getbit(SPI_STATUS(id), F_spi_rx_empty); timeout++);
		if (timeout == 100) {
			return i;
		}
		reg = SPI_DATA(id);
		indata[i] = reg;
		indata[i+1] = reg >> 8;
		indata[i+2] = reg >> 16;
		indata[i+3] = reg >> 24;
	}
	return i;
}

/*
 * NƳzLSPIeX
 */
void SPI_Module::Write(const char *outdata, int len)
{
	uint reg;
	int sent_bytes = 0;

	sent_bytes = WriteFIFO(outdata, len);

	reg = 0;
	setbit(reg, F_spi_write_cnt, len-1);
	setbit(reg, F_spi_transfer_mode, SPI_WRITE);
	setbit(reg, F_spi_spit, 1);
	SPI_DATA_CTRL(id) = reg;

	while (sent_bytes < len) {
		sent_bytes += WriteFIFO(outdata+sent_bytes, len-sent_bytes);
	}
}

/*
 * }SPI slaveQgJ\AmasterݥiHgJƶi
 */
void SPI_Module::WaitForRead(int len)
{
	uint reg;
	reg = 0;
	setbit(reg, F_spi_read_cnt, len-1);
	setbit(reg, F_spi_transfer_mode, SPI_READ);
	setbit(reg, F_spi_spit, 1);
	SPI_DATA_CTRL(id) = reg;
}

/*
 * gŪApGdummylen0AN|Jdummy bytes
 */
int SPI_Module::WriteRead(const char *outdata, int outlen, char *indata, int inlen, int dummylen)
{
	uint reg;

	setbit(SPI_CTRL(id), F_spi_tx_reset, 1);
	setbit(SPI_CTRL(id), F_spi_rx_reset, 1);

	WriteFIFO(outdata, outlen);

	reg = 0;
	setbit(reg, F_spi_write_cnt, outlen-1);
	setbit(reg, F_spi_read_cnt, inlen-1);
	if (dummylen > 0) {
		setbit(reg, F_spi_dummy_cnt, dummylen-1);
		setbit(reg, F_spi_transfer_mode, SPI_WRITE_DUMMY_READ);
	} else {
		setbit(reg, F_spi_transfer_mode, SPI_WRITE_READ);
	}
	setbit(reg, F_spi_spit, 1);
	SPI_DATA_CTRL(id) = reg;

	return Read(indata, inlen);
}




void SPI_Flash_Control::Init(SPI_Module *spi_master)
{
	spi = spi_master;
}

void SPI_Flash_Control::ReadId(char *id)
{
	char cmd = SPI_RDID;
	int len = spi->WriteRead(&cmd, 1, id, 3, 0);

	UART1.printf("SPI_RDID(%d) %02x %02x %02x\n", len, uchar(id[0]), uchar(id[1]), uchar(id[2]));
}

void SPI_Flash_Control::Read(int address, char *data, int length)
{
	char cmd[4] = {SPI_READ_CMD};
	cmd[1] = char(address >> 16);
	cmd[2] = char(address >> 8);
	cmd[3] = char(address);

	int len = spi->WriteRead(cmd, 4, data, length, 0);
	for (int i=0; i<len; i++) {
		UART1.printf("%02x ", uchar(data[i]));
		if (i % 16 == 15) UART1.puts("\n");
	}
	UART1.puts("\n");
}

void SPI_Flash_Control::FastRead(int address, char *data, int length)
{
	char cmd[4] = {SPI_FAST_READ};
	cmd[1] = char(address >> 16);
	cmd[2] = char(address >> 8);
	cmd[3] = char(address);

	int len = spi->WriteRead(cmd, 4, data, length, 1);
	for (int i=0; i<len; i++) {
		UART1.printf("%02x ", uchar(data[i]));
		if (i % 16 == 15) UART1.puts("\n");
	}
	UART1.puts("\n");
}

void SPI_Flash_Control::WriteEn(bool enable)
{
	char cmd = enable ? SPI_WREN : SPI_WRDI;
	spi->Write(&cmd, 1);
}

void SPI_Flash_Control::SectorErase(int address)
{
	char cmd[4] = {SPI_SEC_ERASE};
	cmd[1] = char(address >> 16);
	cmd[2] = char(address >> 8);
	cmd[3] = char(address);

	WriteEn(true);
	spi->Write(cmd, 4);
	WriteEn(false);
}

void SPI_Flash_Control::PageProgram(int address, const char *data, int length)
{
	char cmd[4] = {SPI_PAGE_PROG};
	cmd[1] = char(address >> 16);
	cmd[2] = char(address >> 8);
	cmd[3] = char(address);

	WriteEn(true);
	spi->WriteFIFO(cmd, 4);
	spi->Write(data, length+4);
	WriteEn(false);
}




void I2C_Module::Init(bool masterMode, uchar slaveAddr)
{
	setbit(I2C_TIMING, F_i2c_gsr, 1);
	setbit(I2C_TIMING, F_i2c_tsr, 1);

	uint reg = 0;
	master = masterMode;
	setbit(reg, F_i2c_enable, 1);
	setbit(reg, F_i2c_master_scl, masterMode);
	I2C_CTRL = reg;

	slaveAddress = slaveAddr;
	I2C_SLAVEADDR = slaveAddress >> 1;

	// set frequency to 300khz
	I2C_CLOCK = (CPU_FREQ / 300000 - 5) / 2;

	if (master == false) {
		setbit(INT_MASK1, IRQ_I2C_MASTER, 1);
		setbit(I2C_CTRL, F_i2c_int_slave, 1);
	}
}

inline bool I2C_Module::WaitForStatus(uint mask, uint status)
{
	int timeout;
	for (timeout=0; timeout<100; timeout++) {
		if ((I2C_STATUS & mask) == status) break;
	}
	return (timeout >= 100);
}

void I2C_Module::SlaveProcess()
{
	WaitForStatus(I2C_SLAVEADDR_MASK, I2C_SLAVEADDR_OK);

	if (getbit(I2C_STATUS,F_i2c_readwrite) == 1)	// slave write to master
	{
		for (int i=0; i<256; i++) {
			I2C_DATA = 0xAA;
			setbit(I2C_CTRL, F_i2c_transfer_en, 1);
			if (WaitForStatus(I2C_SLAVEWR_MASK, I2C_SLAVEWR_OK)) break;
		}
	}
	else	// slave read from master
	{
		for (int i=0; i<256; i++) {
			setbit(I2C_CTRL, F_i2c_transfer_en, 1);
			if (WaitForStatus(I2C_SLAVERD_MASK, I2C_SLAVERD_OK)) break;
			UART1.printf("read %02x\n", I2C_DATA);
		}
	}
}

void I2C_Module::Write(const char *data, int n)
{
	if (master == false) return;

	I2C_DATA = data[0];
	setbit(I2C_CTRL, F_i2c_transfer, I2C_START);
	WaitForStatus(I2C_MASTERWR_MASK, I2C_MASTERWR_OK);

	for (int i=1; i<n; i++) {
		I2C_DATA = data[i];
		if (i == n-1) {
			setbit(I2C_CTRL, F_i2c_transfer, I2C_STOP);
		} else {
			setbit(I2C_CTRL, F_i2c_transfer, I2C_TX);
		}
		WaitForStatus(I2C_MASTERWR_MASK, I2C_MASTERWR_OK);
	}
}

void I2C_Module::WriteRead(const char *dataout, int n, char *datain, int inlen)
{
	if (master == false) return;

	I2C_DATA = dataout[0];
	setbit(I2C_CTRL, F_i2c_transfer, I2C_START);
	WaitForStatus(I2C_MASTERWR_MASK, I2C_MASTERWR_OK);

	for (int i=1; i<n; i++) {
		I2C_DATA = dataout[i];
		setbit(I2C_CTRL, F_i2c_transfer, I2C_TX);
		WaitForStatus(I2C_MASTERWR_MASK, I2C_MASTERWR_OK);
	}

	I2C_DATA = dataout[0] | 0x01;					// auto set read bit
	setbit(I2C_CTRL, F_i2c_transfer, I2C_START);	// restart
	WaitForStatus(I2C_MASTERWR_MASK, I2C_MASTERWR_OK);

	for (int i=0; i<inlen; i++) {
		if (i == inlen-1) {
			setbit(I2C_CTRL, F_i2c_transfer, I2C_STOP);
			setbit(I2C_CTRL, F_i2c_ack, 1);
		}
		else {
			setbit(I2C_CTRL, F_i2c_transfer, I2C_TX);
		}
		WaitForStatus(I2C_MASTERRD_MASK, I2C_MASTERRD_OK);
		datain[i] = I2C_DATA;
	}
	setbit(I2C_CTRL, F_i2c_ack, 0);
}

// connect to i2c slave to read from
uint I2C_Module::TestI2CRead (uint address)
{
	char data[3] = {0xf2, 0x00, 0x00};
	uint ret = 0;

	for (int i=0; i<4; i++) {
		data[1] = i;
		data[2] = address >> (i * 8);
		Write(data, 3);
	}
	data[1] = 0x0c;
	data[2] = 0x00;		// read i2c slave
	Write(data, 3);

	for (ret=0; ret<10; ret++)
	{
		char busy, error;
		data[1] = 0x0e;
		WriteRead(data, 2, &busy, 1);
		data[1] = 0x0f;
		WriteRead(data, 2, &error, 1);
		if (!busy && !error) break;
	}
	if (ret == 10) return 0xdeadbeef;

	ret = 0;
	for (int i=0; i<4; i++) {
		data[1] = i+8;
		WriteRead(data, 2, &data[2], 1);
		ret |= uint(uchar(data[2])) << (i*8);
	}
	return ret;
}

bool I2C_Module::TestI2CWrite(uint address, uint data)
{
	char str[3] = {0xf2};

	for (int i=0; i<4; i++) {
		str[1] = i;
		str[2] = address >> (i * 8);
		Write(str, 3);
	}
	for (int i=0; i<4; i++) {
		str[1] = i+4;
		str[2] = data >> (i * 8);
		Write(str, 3);
	}
	str[1] = 0x0c;
	str[2] = 0x01;
	Write(str, 3);

	for (int i=0; i<10; i++)
	{
		char busy, error;
		str[1] = 0x0e;
		WriteRead(str, 2, &busy, 1);
		str[1] = 0x0f;
		WriteRead(str, 2, &error, 1);
		if (!busy && !error) break;
	}
	return false;
}

